from okahu_apptrace.instrumentation.common.utils import resolve_from_alias
import logging
logger = logging.getLogger(__name__)

def handle_openai_response(response):
    try:
        if hasattr(response,'__stream__'):
            return str(response.response.headers)
        else:
            return response.choices[0].message.content
    except Exception as e:
        logger.warning("Warning: Error occurred in handle_openai_response: %s", str(e))
        return ""

def agent_instructions(arguments):
    if callable(arguments['kwargs']['agent'].instructions):
        return arguments['kwargs']['agent'].instructions(arguments['kwargs']['context_variables'])
    else:
        return arguments['kwargs']['agent'].instructions

def extract_input(arguments):
    history = arguments['kwargs']['history']
    content = next((entry['content'] for entry in reversed(history) if entry['role'] == 'user'), None)
    return content

def get_inference_endpoint(arguments):
    inference_endpoint = resolve_from_alias(arguments['instance'].client.__dict__, ['azure_endpoint', 'api_base', '_base_url'])
    return str(inference_endpoint)

def update_span_from_llm_response(response):
    meta_dict = {}
    token_usage = None
    if response is not None and hasattr(response, "usage"):
        if hasattr(response, "usage") and response.usage is not None:
            token_usage = response.usage
        if token_usage is not None:
            meta_dict.update({"completion_tokens": getattr(token_usage, 'completion_tokens', None)})
            meta_dict.update({"prompt_tokens": getattr(token_usage, 'prompt_tokens', None)})
            meta_dict.update({"total_tokens": getattr(token_usage, 'total_tokens', None)})
    return meta_dict
